import { useMemo } from "react";
import {
  useExpressionCategories,
  useExpressionFilterQueryIds,
  useExpressionSetQueryId,
  useExpressionsInSet,
} from "../../hooks";
import { IndexedExpression } from "../../model";
import { sample } from "../../util";
import { ErrorView } from "../ErrorView";
import { ExpressionPracticeCardView } from "./ExpressionPracticeCardView";

export function ExpressionPracticeView() {
  const expression_set_id = useExpressionSetQueryId();
  const filter_ids = useExpressionFilterQueryIds();
  const expressions = useExpressionsInSet(expression_set_id);

  // Fallback rendering for content not yet fetched
  if (!expressions) return null;
  if (!filter_ids) return null;

  // Fallback views for expression set content not found
  if (!expression_set_id) {
    return <ErrorView message="Expression set not found" />;
  }
  const filtered_expressions = expressions.filter(
    (expression) => !filter_ids.includes(expression.id!)
  );
  if (!filtered_expressions.length) {
    return <ErrorView message="No expressions left in this set" />;
  }

  return (
    <ExpressionPracticeViewImpl
      select_from_expressions={filtered_expressions}
    />
  );
}

interface ExpressionPracticeViewImplProps {
  select_from_expressions: IndexedExpression[];
}

function ExpressionPracticeViewImpl({
  select_from_expressions: expressions,
}: ExpressionPracticeViewImplProps) {
  const expression = useMemo(() => sample(expressions), [expressions]);
  const categories = useExpressionCategories(expression.id!);
  if (!categories) return null; // Loading

  // Delegate internal interaction state to next component so it resets
  // on an expression switch
  return (
    <ExpressionPracticeCardView
      key={expression.id}
      expression={expression}
      categories={categories}
    />
  );
}
